/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <X11/Xos.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>

#include <stdio.h>
#include <time.h>
#include <errno.h>
#include "xmap.h"
#include "gps.h"
#include "debug.h"

extern Widget status_3;
XmFontList fontlist;

static slen = 0;

double cv_dist(double dist)
{
    switch (unit) {
    case NAUTICAL:
	dist /= KM2NMILES;
	break;

    case ENGLISH:
	dist /= KM2MILES;
	break;

    default:
	//METRIC
	break;
    }
    return dist;
}


static long get_raw_distance(int pix)
{
    double dist;

#if 1
    double lat1, lat2, lon1, lon2;

    xytolatlon( 100, 100, &lat1, &lon1 );
    xytolatlon( 100+pix, 100, &lat2, &lon2 );
    dist = distance(lat1, lon1, lat2, lon2);
#else
    //
    // This does NOT work with vector maps!
    //
    dist = distance(mapdata.lat1, mapdata.lon1, mapdata.lat1,
		    mapdata.lon1 + ((double)pix * (double)mapdata.zoom) / mapdata.pixd_x);
#endif

    dist = cv_dist(dist);
    return (long) (10.0 * dist);
}

void draw_scale(void)
{
    Dimension width, height;
    XmString s;
    long res1, res2;
    int max;
    char *p;
    extern long vscale();
    char bufp[256];

    if (!status_3)
	return;

    XtVaGetValues(status_3,
		  XmNwidth, &width,
		  XmNheight, &height,
		  NULL);

    XClearArea(XtDisplay(status_3), XtWindow(status_3),
	       0, 0, width, height, False);

    if (mapdata.wlat > 20.0) return;


    slen = XTextWidth(
	     XQueryFont(XtDisplay(status_3), XGContextFromGC(status3GC)),
			 "WWWWWWW", 7);

    max = width - slen - 20;


    res1 = (long) get_raw_distance(max);
// fprintf(stderr,"res1=%ld max=%ld\n",res1, max);

    if (res1 > 10000) {
	res2 = (res1 / 1000) * 1000;
    } else if (res1 > 1000) {
	res2 = (res1 / 150) * 150;
    } else if (res1 > 150) {
	res2 = (res1 / 10) * 10;
    } else if (res1 > 100) {
	res2 = (res1 / 10) * 10;
    } else if (res1 > 10) {
	res2 = (res1 / 10) * 10;
    } else
	res2 = res1;

    max = (int) (((double) res2 / (double) res1) * (double) max);

    XDrawLine(XtDisplay(status_3), XtWindow(status_3), status3GC,
	      5, height / 2 - 3, 5, height / 2 + 3);
    XDrawLine(XtDisplay(status_3), XtWindow(status_3), status3GC,
	      5 + max, height / 2 - 3, 5 + max, height / 2 + 3);
    XDrawLine(XtDisplay(status_3), XtWindow(status_3), status3GC,
	      5, height / 2, 5 + max, height / 2);

    switch (unit) {
    case METRIC:
	p = "Km";
	break;
    case NAUTICAL:
	p = "nm";
	break;
    case ENGLISH:
	p = "mi";
	break;
    }

    snprintf(bufp, 255, "%.1lf %s", ((double) res2) / 10.0, p);

    XDrawString(XtDisplay(status_3), XtWindow(status_3),
		status3GC, max + 15, 11, bufp, strlen(bufp));
}

void st3_redraw_cb()
{
    if (mapdata.calibrated)
	draw_scale();
}
